﻿Configuration DSCHyperV_VM {
  param (
        # Zielsystem auf dem die VM erzeugt werden soll
        [Parameter()]
        [string[]]$NodeName = 'localhost',

        # Pfad auf dem Zielsystem, unter dem die VM erzeugt werden soll
        [Parameter()]
        [string]$HypervPath = 'E:\Hyper-V',

        # Name der VM
        [Parameter(Mandatory)]
        [string]$VMName,

        # Pfadname des Basis Image der VM
        [Parameter(Mandatory)]
        [string]$ParentVhdPath,

        # Kopie des Basis Image oder Diffencing Disk erstellen 
        [ValidateSet("Differencing","Copy")]
        [string]$OSDiskType = "Differencing",

        # Hyper-V Switch, mit dem die VM verbunden werden soll
        [Parameter()]
        [string]$SwitchName = 'NatSwitch80',

        # Startup RAM der VM
        [ValidateRange(1,8)]
        [UInt64]$ProcessorCount = 2,

        # Startup RAM der VM
        [ValidateRange(512MB,16GB)]
        [UInt64]$StartupMemory = 2GB,

        # Minimum RAM der VM. Hierdurch wird dynamic RAM aktiviert
        [ValidateRange(512MB,16GB)]
        [UInt64]$MinimumMemory = 1GB,

        # Maximum RAM der VM. Hierdurch wird dynamic RAM aktiviert
        [ValidateRange(2GB,16GB)]
        [UInt64]$MaximumMemory = 16GB
    )

#region - Importieren benötigter DSC Ressourcen 
    Import-DscResource -ModuleName 'PSDesiredStateConfiguration'
    Import-DscResource -module xHyper-V -ModuleVersion 3.6.0.0
    Import-DscResource -module cHyper-V -ModuleVersion 3.0.0.0
#endregion

#region - Variablendefinitionen 
    # Einige Variable
    $VMPath = Join-Path -Path $HypervPath -ChildPath $VMName            # Verzeichnispfad für die VM
    $VhdPath = Join-Path -Path $VMPath  -ChildPath "Virtual Hard Disks" # Verzeichnispfad für die virt. Disks der VM
    $VhdFilePath = Join-Path -Path $VhdPath -ChildPath "$VMname.vhdx"   # Pfadname der Boot Disk
    $VMSetupFilePath = Join-Path -Path (Get-Location).Path -ChildPath "$VMName\Setup\*" # Setup Dateien für die VM
#endregion

#region - Ressourcenblöcke zum Erzeugen einer VM 
    Node $NodeName
    {
        # Das Verzeichnis für die VM erzeugen
        File VMPath
        {
            Ensure = 'Present'  
            Type = 'Directory'
            DestinationPath = $VMPath
        }
        # Das Verzeichnis für die virtuellen Disks der VM erzeugen
        File VHDPath
        {
            Ensure = 'Present'  
            Type = 'Directory'
            DestinationPath = $VhdPath
            DependsOn = '[File]VMPath'
        }

        # Soll die VM aus einer Kopie oder aus einer Differential Disk starten
        if ( $OSDiskType -eq "Copy") 
        {
            # Kopie des Basis Image im Disk Verzeichnis erstellen
            File BootVHD
            {
                SourcePath = $ParentVhdPath
                DestinationPath = $VhdFilePath
                Ensure = 'Present'
                DependsOn = '[File]VHDPath'
            }
        }
        else
        {
            # Differential Disk erzeugen
            xVHD BootVHD 
            {
                Ensure = 'Present'
                Name = $VMName
                Path = $VhdPath
                ParentPath = $ParentVhdPath
                Generation = 'vhdx'
                DependsOn = '[File]VHDPath'
            }
        }

        # copy setup files for VM if available
        if ( Test-Path $VMSetupFilePath )
        {
            xVHDFile SetupFiles
            {
                VhdPath = $VhdFilePath
                FileDirectory = MSFT_xFileDirectory {
                                   SourcePath = $VMSetupFilePath
                                   DestinationPath = "."
                                   }
            }
        }

        # Die VM im Hyper-V anlegen
        xVMHyperV CreateVM 
        {
            Name = $VMName
            SwitchName = $SwitchName
            VhdPath = Join-Path -Path $VhdPath -ChildPath "$VMname.vhdx"
            ProcessorCount = 2
            StartupMemory = $StartupMemory
            MaximumMemory = $MaximumMemory
            MinimumMemory = $MinimumMemory
            State = 'Running'
            Generation = 2
            # EnableGuestService = $true
            Path = $HypervPath
        }
        # Entfernen des Standard Netzwerkadapters und Neuanlegen mit vernünftigen Namen
        cVMNetworkAdapter StdNICen
        {
            Id = ([guid]::NewGuid()).guid
            SwitchName = $SwitchName
            Name = 'Network Adapter'         # Name des virt. Netzwerkadapters in englischen OS
            VMName = $VMName
            Ensure = 'Absent'
            DependsOn = '[xVMHyperV]CreateVM'
        }
        cVMNetworkAdapter StdNICde
        {
            Id = ([guid]::NewGuid()).guid
            SwitchName = $SwitchName
            Name = 'Netzwerkkarte'         # Name des virt. Netzwerkadapters in deutschen OS
            VMName = $VMName
            Ensure = 'Absent'
            DependsOn = '[xVMHyperV]CreateVM'
        }
        cVMNetworkAdapter Mgmt
        {
            Id = ([guid]::NewGuid()).guid
            Name = 'Mgmt'
            SwitchName = $SwitchName
            VMName = $VMName
            Ensure = 'Present'
            DependsOn = '[xVMHyperV]CreateVM'
        }
        cVMNetworkAdapterSettings Mgmt
        {
            Id = ([guid]::NewGuid()).guid
            Name = 'Mgmt'
            VMName = $VMName
            SwitchName = $SwitchName
            DeviceNaming = 'On'            # Nur bei Server 2016 / Windows 10 V 1607
            DependsOn = '[cVMNetworkAdapter]Mgmt'
        }
    }
#endregion
}
