﻿Param
(
    [Parameter()]
	[String]$VMname = 'DSC-PC01',
    [Parameter()]
	[String]$InterfaceAlias = 'Mgmt',
    [Parameter()]
	[String]$IPAddress = '192.168.80.251',
    [Parameter()]
	[String]$DefaultGateway = '192.168.80.1',
    [Parameter()]
	[String]$PrefixLebgth = '24',
    [Parameter()]
	[String]$AddressFamily = 'IPv4',
    [Parameter()]
	[String]$DNSAddress = '192.168.1.1'
)

#region - SetupComplete.cmd ########################################
# In dieses Verzeichnis kommt die Batchdatei SetupComplete.Cmd, 
# über die die VM-spezifische Konfiguration gestartet wird.
# Sie wird am Ende einer Windows Installation automatisch aufgerufen
#
$SetupCompleteFolder = Join-Path $PSScriptRoot -ChildPath "$VMname\Setup\Windows\Setup\Scripts"
mkdir -Path $SetupCompleteFolder -Force
#
# Und nachdem SetupCompelte.cmd für alle VMs identisch ist, können wir sie gleich erzeugen
#
$SetupCompleteFile = New-Item "$SetupCompleteFolder\SetupComplete.cmd" -type File -Force
$fileContent = @"
@echo off
PowerShell.exe -ExecutionPolicy RemoteSigned -File C:\DSC\RunDSC.ps1
exit
"@
Set-Content -path $SetupCompleteFile -value $fileContent
###############################################################
#endregion

#region - DSC folder with configuration data / scripts
#### RunDSC.ps1 ###############################################
# In dieses Verzeichnis kommen alle VM-spezifischen Konfigurationsdaten
$DSCFolder = Join-Path $PSScriptRoot -ChildPath "$VMname\Setup\DSC"
mkdir -Path $DSCFolder -Force
#
# Und nachdem einige der Dateien für alle VMs identisch sind, können wir sie 
# entweder gleich erzeugen oder zumindest ein Template dafür anlegen
#

#region - RunDSC.ps1
$RunDSCFile = New-Item "$DSCFolder\RunDSC.ps1" -type File -Force
$fileContent = @'
# RunDSC.ps1 - finish system setup with DSC

Set-Location -Path $PSScriptRoot

# 1. rename network adapters to their DisplayValues 
#    given by Hyper-V device naming property
.\Rename-NetAdapterToHyperVDeviceNaming.ps1

# 2. configure LCM for the partial configurations
.\LCMconfig.ps1
Set-DscLocalConfigurationManager -path .\LCMconfig -Force

# 3. Apply partial DSC Configurations

# 3.1. define OS configuration (IP addresses and computername)
.\OSconfig.ps1 -ConfigurationData .\OSconfig-data.psd1 -OutputPath .\OSconfig
Publish-DscConfiguration  -Force -Verbose -Path .\OSconfig

# 3.2. APPconfig will be pulled from Pull Server

# 3.3. start the configuration
Start-DscConfiguration -Wait -Force -Verbose -UseExisting
'@
Set-Content -path $RunDSCFile -value $fileContent
#endregion

#region - Rename-NetAdapterToHyperVDeviceNaming.ps1 
$RenNICFile = New-Item "$DSCFolder\Rename-NetAdapterToHyperVDeviceNaming.ps1" -type File -Force
$fileContent = @'
# rename network adapters to their DisplayValues given by Hyper-V device naming property
Get-NetAdapterAdvancedProperty | where Displayname –eq “Hyper-V Network Adapter Name” | select Name, DisplayValue | % { 
    Write-Host 'Renaming Network Adapter '$_.Name' to '$_.DisplayValue
    Rename-NetAdapter -Name $_.Name -NewName $_.DisplayValue 
    }
'@
Set-Content -path $RenNICFile -value $fileContent
#endregion

#region - LCMconfig.ps1
$LCMconfigFile = New-Item "$DSCFolder\LCMconfig.ps1" -Type File -Force
$fileContent = @'
[DSCLocalConfigurationManager()]
configuration LCMconfig
{
    Node localhost
    {
        Settings
        {
            RefreshMode = 'Push'
            AllowModuleOverwrite = $true
            DebugMode = 'All'
            RebootNodeIfNeeded = $true
            ActionAfterReboot = 'ContinueConfiguration'
            RefreshFrequencyMins = 30
            ConfigurationModeFrequencyMins = 60
            ConfigurationMode = 'ApplyAndAutoCorrect'
        }
        ConfigurationRepositoryWeb DSC-PS01
        {
            ServerURL = 'http://DSC-PS01:8080/PSDSCPullServer.svc'
            RegistrationKey = '14fc8e72-5036-4e79-9f89-5382160053aa'
            AllowUnsecureConnection = $true
            ConfigurationNames = @('EnableMSUpdate')
        }
        ReportServerWeb DSCC-PS01
        {
            ServerURL = 'http://DSC-PS01:8080/PSDSCPullServer.svc'
            AllowUnsecureConnection = $true
        }

        PartialConfiguration OSConfig
        {
            Description = 'Configure OS with computername and IP address.'
            RefreshMode = 'Push'
        }
        PartialConfiguration EnableMSUpdate
        {
            Description = 'Configure VM for NS Update.'
            RefreshMode = 'Pull'
            ConfigurationSource = @("[ConfigurationRepositoryWeb]DSC-PS01")
            DependsOn = '[PartialConfiguration]OSconfig'
        }
    }
}
LCMconfig
'@
Set-Content -path $LCMconfigFile -value $fileContent
#endregion

#region - OSconfig.ps1
$OSconfigFile = New-Item "$DSCFolder\OSconfig.ps1" -type File -Force
$fileContent = @'
Configuration OSconfig {

    Import-DscResource –Module PSDesiredStateConfiguration
    Import-DscResource -Module xComputerManagement -ModuleVersion 1.9.0.0
    Import-DscResource -Module xNetworking -ModuleVersion 3.1.0.0

    Node $AllNodes.NodeName {
        xIPAddress SetIP {
            IPAddress = $Node.IPAddress
            InterfaceAlias = $Node.InterfaceAlias
            PrefixLength = $Node.PrefixLength
            AddressFamily = $Node.AddressFamily
        }
        xDefaultGatewayAddress SetDefaultGateway {
            Address        = $node.DefaultGateway
            InterfaceAlias = $Node.InterfaceAlias
            AddressFamily  = $Node.AddressFamily
        }
        xDNSServerAddress SetDNS {
            Address = $Node.DNSAddress
            InterfaceAlias = $Node.InterfaceAlias
            AddressFamily = $Node.AddressFamily
        }
       xComputer SetName {
            Name = $Node.MachineName
        }
    }
}
OSconfig -ConfigurationData OSconfig-data.psd1
'@
Set-Content -path $OSconfigFile -value $fileContent
#endregion

#region - OSconfig-data.psd1
$OSconfigDataFile = New-Item "$DSCFolder\OSconfig-data.psd1" -type File -Force
$fileContent = @"
@{
    AllNodes = @(
        @{
            NodeName = 'localhost'
            MachineName = 'DSC-PC01'
            InterfaceAlias = 'Mgmt'
            AddressFamily = 'IPv4'
            IPAddress = '192.168.80.251'
            DefaultGateway = '192.168.80.1'
            PrefixLength = '24'
            DNSAddress = '192.168.80.10','192.168.1.1'
        }
    )
}
"@
Set-Content -path $OSconfigDataFile -value $fileContent
#endregion

#region - APPconfig.ps1
# - APPconfig will be pulled from DSC-PS01
#endregion

#endregion

#region - Kopieren benötigter DSC Ressourcen
# In dieses Verzeichnis kommen alle VM-spezifischen DSC Ressourcen
$DestDSCResFolder = Join-Path $PSScriptRoot -ChildPath "$VMname\Setup\Program Files\WindowsPowerShell\Modules"
mkdir -Path $DestDSCResFolder -Force

# DSC Ressourcen, die in der VM benörigt werden, aus dem Host kopieren
$SrcDSCResFolder = Join-Path $env:ProgramFiles -ChildPath "WindowsPowerShell\Modules"
Copy-Item -Path (Join-Path $SrcDSCResFolder -ChildPath 'xPSDesiredStateConfiguration') -Destination $DestDSCResFolder -Recurse -Force
Copy-Item -Path (Join-Path $SrcDSCResFolder -ChildPath 'xComputerManagement') -Destination $DestDSCResFolder -Recurse -Force
Copy-Item -Path (Join-Path $SrcDSCResFolder -ChildPath 'xNetworking') -Destination $DestDSCResFolder -Recurse -Force

# To Do: Kopieren weiterer DSC Ressourcen, z.B. für Anwendungen und Serverrollen

#endregion
